@@ -59,7 +59,7 @@ use base qw( Exporter DynaLoader );
 sub dl_load_flags { 0x01 } # global option
 
 BEGIN {
-  $VERSION = '1.07';
+  $VERSION = '2.00';
   bootstrap Net::Silk $VERSION;
 }
 
@@ -67,40 +67,47 @@ use constant DEBUG => 0;
 
 module_init($FindBin::Script);
 
-use constant SILK_IPWILDCARD_CLASS => 'Net::Silk::IPWildcard';
-
-use constant SILK_IPADDR_CLASS     => 'Net::Silk::IPAddr';
-use constant SILK_IPV4ADDR_CLASS   => 'Net::Silk::IPv4Addr';
-use constant SILK_IPV6ADDR_CLASS   => 'Net::Silk::IPv6Addr';
-
-use constant SILK_PROTOPORT_CLASS => 'Net::Silk::ProtoPort';
-
-use constant SILK_TCPFLAGS_CLASS => 'Net::Silk::TCPFlags';
-
-use constant SILK_RWREC_CLASS => 'Net::Silk::RWRec';
-
-use constant SILK_IPSET_CLASS  => 'Net::Silk::IPSet';
-use constant SILK_BAG_CLASS    => 'Net::Silk::Bag';
-
-use constant SILK_PMAP_CLASS    => 'Net::Silk::Pmap';
-use constant SILK_PMAP_IP_CLASS => 'Net::Silk::Pmap::IP';
-use constant SILK_PMAP_PP_CLASS => 'Net::Silk::Pmap::PP';
-
+use constant SILK_IPWILDCARD_CLASS      => 'Net::Silk::IPWildcard';
+use constant SILK_IPADDR_CLASS          => 'Net::Silk::IPAddr';
+use constant SILK_IPV4ADDR_CLASS        => 'Net::Silk::IPv4Addr';
+use constant SILK_IPV6ADDR_CLASS        => 'Net::Silk::IPv6Addr';
+use constant SILK_RANGE_CLASS           => 'Net::Silk::Range';
+use constant SILK_RANGE_IP_CLASS        => 'Net::Silk::Range::IP';
+use constant SILK_RANGE_PP_CLASS        => 'Net::Silk::Range::ProtoPort';
+use constant SILK_CIDR_CLASS            => 'Net::Silk::CIDR';
+use constant SILK_PROTOPORT_CLASS       => 'Net::Silk::ProtoPort';
+use constant SILK_TCPFLAGS_CLASS        => 'Net::Silk::TCPFlags';
+use constant SILK_RWREC_CLASS           => 'Net::Silk::RWRec';
+use constant SILK_IPSET_CLASS           => 'Net::Silk::IPSet';
+use constant SILK_BAG_CLASS             => 'Net::Silk::Bag';
+use constant SILK_PMAP_CLASS            => 'Net::Silk::Pmap';
+use constant SILK_PMAP_IPV4_CLASS       => 'Net::Silk::Pmap::IPv4';
+use constant SILK_PMAP_IPV6_CLASS       => 'Net::Silk::Pmap::IPv6';
+use constant SILK_PMAP_PP_CLASS         => 'Net::Silk::Pmap::ProtoPort';
 use constant SILK_IPWILDCARD_ITER_CLASS => 'Net::Silk::IPWildcard::iter_xs';
 use constant SILK_IPSET_ITER_CLASS      => 'Net::Silk::IPSet::iter_xs';
 use constant SILK_BAG_ITER_CLASS        => 'Net::Silk::Bag::iter_xs';
 use constant SILK_PMAP_ITER_CLASS       => 'Net::Silk::Pmap::iter_xs';
 use constant SILK_SITE_REPO_ITER_CLASS  => 'Net::Silk::Site::iter_xs';
-
-use constant SILK_SITE_CLASS    => 'Net::Silk::Site';
-use constant SILK_FILE_CLASS    => 'Net::Silk::File';
-use constant SILK_FILE_IO_CLASS => 'Net::Silk::File::io_xs';
+use constant SILK_SITE_CLASS            => 'Net::Silk::Site';
+use constant SILK_FILE_CLASS            => 'Net::Silk::File';
+use constant SILK_FILE_IO_CLASS         => 'Net::Silk::File::io_xs';
 
 use Math::Int64;
 use Math::Int64::die_on_overflow;
-use Math::Int128;
+use Math::Int128 qw(string_to_uint128);
 use Math::Int128::die_on_overflow;
 
+my $IPv6Max;
+BEGIN {
+  $IPv6Max = string_to_uint128("0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF");
+  $IPv6Max = SILK_IPV6ADDR_CLASS->new($IPv6Max) if SILK_IPV6_ENABLED;
+}
+use constant SILK_IPV6ADDR_MAX  => $IPv6Max;
+use constant SILK_IPV6ADDR_BITS => 128;
+use constant SILK_IPV4ADDR_MAX  => SILK_IPV4ADDR_CLASS->new(0xFFFFFFFF);
+use constant SILK_IPV4ADDR_BITS => 32;
+
 # note: SILK_IPV6_ENABLED defined in Silk.xs/BOOT
 
 BEGIN {
@@ -113,6 +120,11 @@ BEGIN {
     SILK_LOCALTIME_ENABLED
     SILK_INITIAL_TCPFLAGS_ENABLED
 
+    SILK_IPV4ADDR_MAX
+    SILK_IPV4ADDR_BITS
+    SILK_IPV6ADDR_MAX
+    SILK_IPV6ADDR_BITS
+
     SILK_IPWILDCARD_CLASS
     SILK_IPWILDCARD_ITER_CLASS
 
@@ -131,10 +143,16 @@ BEGIN {
     SILK_BAG_ITER_CLASS
 
     SILK_PMAP_CLASS
-    SILK_PMAP_IP_CLASS
+    SILK_PMAP_IPV4_CLASS
+    SILK_PMAP_IPV6_CLASS
     SILK_PMAP_PP_CLASS
     SILK_PMAP_ITER_CLASS
 
+    SILK_RANGE_CLASS
+    SILK_RANGE_IP_CLASS
+    SILK_RANGE_PP_CLASS
+    SILK_CIDR_CLASS
+
     SILK_SITE_REPO_ITER_CLASS
 
     SILK_PROTOPORT_CLASS
@@ -180,15 +198,18 @@ END { module_destroy() }
 
 ###
 
-require Net::Silk::IPWildcard;
-require Net::Silk::Site;
-require Net::Silk::IPAddr;
+require Net::Silk::RWRec;
 require Net::Silk::IPSet;
-require Net::Silk::ProtoPort;
-require Net::Silk::Pmap;
-require Net::Silk::File;
 require Net::Silk::Bag;
+require Net::Silk::Pmap;
+require Net::Silk::IPWildcard;
+require Net::Silk::Range;
+require Net::Silk::CIDR;
+require Net::Silk::IPAddr;
 require Net::Silk::TCPFlags;
+require Net::Silk::ProtoPort;
+require Net::Silk::File;
+require Net::Silk::Site;
 
 ###
 
@@ -214,7 +235,7 @@ is self-described as:
   ideally suited for analyzing traffic on the backbone or border of a
   large, distributed enterprise or mid-sized ISP.
 
-The SiLK suite can be L<found here:|https://tools.netsa.cert.org/silk/index.html>
+The SiLK suite can be L<found here.|https://tools.netsa.cert.org/silk/index.html>
 
 =head1 EXPORTS
 
@@ -249,11 +270,11 @@ Returns either "UTC" or "local" depending on how SiLK was compiled.
 
 =head1 SEE ALSO
 
-L<Net::Silk::RWRec>, L<Net::Silk::IPSet>, L<Net::Silk::IPWildcard>, L<Net::Silk::Bag>, L<Net::Silk::Pmap>, L<Net::Silk::IPAddr>, L<Net::Silk::TCPFlags>, L<Net::Silk::ProtoPort>, L<Net::Silk::File>, L<Net::Silk::Site>, L<silk(7)>
+L<Net::Silk::RWRec>, L<Net::Silk::IPSet>, L<Net::Silk::Bag>, L<Net::Silk::Pmap>, L<Net::Silk::IPWildcard>, L<Net::Silk::Range>, L<Net::Silk::CIDR>, L<Net::Silk::IPAddr>, L<Net::Silk::TCPFlags>, L<Net::Silk::ProtoPort>, L<Net::Silk::File>, L<Net::Silk::Site>, L<silk(7)>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright (C) 2011-2015 by Carnegie Mellon University
+Copyright (C) 2011-2016 by Carnegie Mellon University
 
 Use of the Net-Silk library and related source code is subject to the
 terms of the following licenses:
