@@ -57,9 +57,11 @@ use Math::Int128::die_on_overflow;
 use Math::BigInt;
 
 use Symbol();
+use Scalar::Util qw( refaddr );
 
 use overload (
   '&'    => \&_and,
+  '+'    => \&_and,
   '|'    => \&_or,
   '-'    => \&_sub,
   '^'    => \&_sym,
@@ -82,10 +84,10 @@ use overload (
   'eq'   => \&_eq,
   'ne'   => \&_ne,
   #'0+'   => \&cardinality,
-  '""'   => \&_str,
   'bool' => \&_bool,
   '<>'   => \&_fh_iter,
   '%{}'  => \&_me_hash,
+  '""'   => sub { refaddr shift },
 );
 
 my %Attr;
@@ -98,18 +100,21 @@ sub _parse_set {
 }
 
 sub _parse_item {
-  my $item = $_[0];
-  if (! ref $item) {
-    eval { $item = SILK_IPADDR_CLASS->new($item) };
+  my $item;
+  if (! ref $_[0]) {
+    eval { $item = SILK_IPADDR_CLASS->new($_[0]) };
     if ($@) {
-      eval { $item = SILK_IPWILDCARD_CLASS->new($item) };
-      croak "ip address, string, or number required: $item" if $@;
+      eval { $item = SILK_IPWILDCARD_CLASS->new($_[0]) };
     }
   }
-  elsif (UNIVERSAL::isa($item, 'ARRAY')) {
-    $item = __PACKAGE__->new(@$item);
+  elsif (UNIVERSAL::isa($_[0], 'ARRAY')) {
+    $item = __PACKAGE__->new(@{$_[0]});
   }
-  $item;
+  else {
+    # SILK_IPRANGE_CLASS or SILK_IPCIDR_CLASS
+    eval { $item = $_[0]->as_str };
+  }
+  $item ||= $_[0];
 }
 
 sub new {
@@ -331,49 +336,120 @@ sub symmetric_difference {
 
 sub iter {
   my $self = shift;
-  my $cidr = shift || 0;
-  my $iter = Net::Silk::IPSet::iter_xs->bind($self, $cidr);
-  if ($cidr) {
-    # workaround for libsilk wildcard bug
-    return sub { } unless $self->cardinality;
-    return sub {
-      if (wantarray) {
-        my @items;
-        while (my @block = Net::Silk::IPSet::iter_xs::next($iter)) {
-          push(@items, \@block);
-        }
-        return @items;
+  my $iter = Net::Silk::IPSet::iter_xs->bind($self, 0);
+  return sub {
+    if (wantarray) {
+      # keep $self in scope to prevent premature destruction
+      no warnings;
+      $self;
+      my @items;
+      while (my $item = Net::Silk::IPSet::iter_xs::next($iter)) {
+        push(@items, $item);
       }
-      my @block = Net::Silk::IPSet::iter_xs::next($iter);
-      @block ? \@block : ();
-    };
-  }
-  else {
-    return sub {
-      if (wantarray) {
-        my @items;
-        while (my $item = Net::Silk::IPSet::iter_xs::next($iter)) {
-          push(@items, $item);
-        }
-        return @items;
-      }
-      Net::Silk::IPSet::iter_xs::next($iter) || return;
-    };
-  }
+      return @items;
+    }
+    Net::Silk::IPSet::iter_xs::next($iter) || return;
+  };
 }
 
-sub iter_cidr { shift->iter(1) }
+sub iter_cidr {
+  my $self = shift;
+  my $iter = Net::Silk::IPSet::iter_xs->bind($self, 1);
+  # workaround for libsilk wildcard bug
+  return sub {
+    if (wantarray) {
+      my @items;
+      while (my @block = Net::Silk::IPSet::iter_xs::next($iter)) {
+        push(@items, SILK_CIDR_CLASS->new(@block));
+      }
+      return @items;
+    }
+    # keep $self in scope to prevent premature destruction
+    no warnings;
+    $self;
+    my @block = Net::Silk::IPSet::iter_xs::next($iter);
+    @block ? SILK_CIDR_CLASS->new(@block) : ();
+  };
+}
+
+sub iter_ranges {
+  my $self = shift;
+  my $iter = $self->iter_cidr;
+  my($current, $tmp);
+  my $range_iter = sub {
+    while (my $cidr = $iter->()) {
+      if (!$current) {
+        $current = $cidr->as_range;
+        next;
+      }
+      else {
+        my $first = $cidr->[0];
+        if ($first - $current->[1] == 1) {
+          $current->[1] = $cidr->last;
+          next;
+        }
+        else {
+          $tmp = $current;
+          $current = $cidr->as_range;
+          return $tmp;
+        }
+      }
+    }
+    return $current if $current;
+  };
+  sub {
+    if (wantarray) {
+      my @ranges;
+      while (my $r = $range_iter->()) {
+        push(@ranges, $r);
+      }
+      return @ranges;
+    }
+    else {
+      while (my $r = $range_iter->()) {
+        return $r;
+      }
+    }
+  };
+}
 
 sub iter_bag {
   my $self = shift;
   my $iter = Net::Silk::IPSet::iter_xs->bind($self, 0);
   my $item;
   sub {
-    my $item = Net::Silk::IPSet::iter_xs::next($iter) || return;
-    [$item, 1];
+    if (wantarray) {
+      my @counts;
+      while (my $ip = $iter->()) {
+        push(@counts, [$ip, 1]);
+      }
+      return @counts;
+    }
+    else {
+      my $item = Net::Silk::IPSet::iter_xs::next($iter) || return;
+      return [$item, 1];
+    }
   };
 }
 
+sub as_bag {
+  my $self = shift;
+  my $bag = Net::Silk::Bag->new_ipaddr;
+  $bag->add($self);
+  $bag;
+}
+
+sub bounds {
+  my $self = shift;
+  my($first_addr, $last_block);
+  my $iter = $self->iter_cidr;
+  while (my $block = $iter->()) {
+    $first_addr ||= $block->[0];
+    $last_block = $block;
+  }
+  return($first_addr, $last_block->last, $self->cardinality);
+}
+
 ###
 
 sub is_subset { shift->difference(@_) ? 0 : 1 }
@@ -431,8 +507,6 @@ sub _subu {
   $l->difference_update($r);
 }
 
-sub _str { overload::StrVal($_[0]) }
-
 sub _bool { defined shift->iter->() ? 1 : 0 }
 
 sub _eq {
@@ -570,6 +644,43 @@ Net::Silk::IPSet - SiLK IP sets
 
   use Net::Silk::IPSet;
 
+  $s1 = Silk::IPSet->new(["1.2.3.4", "5.6.7.8"]);
+  $s2 = Silk::IPSet->new;
+  $s2->add("5.6.7.8");
+  $s2 |= "9.10.11.12";
+
+  scalar keys %$s1; # 2
+  $s2->cardinality; # 2
+
+  $s1->is_subset($s2);   # false
+  $s1->is_superset($s2); # false
+  $s1->is_disjoint($s2); # false
+
+  $s3 = $s1 & $s2; # $s1->intersection($s2), 1 element
+  $s3 = $s1 | $s2; # $s1->union($s2), 3 elements
+
+  $s3->{"1.2.3.4"}; # $s1->contains("1.2.3.4"), true
+
+  @ip_list = <$s3>;
+  @ip_list = keys %$s3;
+
+  while ($ip = <$s3>) {
+    print "ip: $ip\n";
+  }
+
+  $iter = $s3->iter_cidr;
+  while ($cidr = $iter->()) {
+    print join('/', @$cidr), "\n";
+  }
+
+  $iter = $s3->iter_cidr_str;
+  while ($cidr = $iter->()) {
+    print "$cidr\n";
+  }
+
+  $s3->clear;
+  $s3->cardinality; # 0
+
 =head1 DESCRIPTION
 
 C<Net::Silk::IPSet> objects represent a set of IP addresses, as produced
@@ -585,7 +696,8 @@ The I<items> can be any of the following:
   * Net::Silk::IPSet
   * string representation of a CIDR block (x.x.x.x/n)
   * string representation of a range (x.x.x.x-y.y.y.y)
-  * ref to array containing any of the above
+  * ref to array of any of the above
+  * arbitrary list of any of the above (multiple arguments)
 
 The following methods are available:
 
@@ -640,10 +752,10 @@ I<item> list.
 Return true if this set is a superset of the union of the given
 I<item> list.
 
-=item is_disjoint($item)
+=item is_disjoint(...)
 
 Return true if this set has no members in common with the union
-of the given I<item> list.
+of the given I<items>.
 
 =item difference(...)
 
@@ -655,32 +767,44 @@ not in the union of the given I<item> list.
 Return a new C<Net::Silk::IPSet> representing IP addresses present in
 both this set and the union of the given I<item> list.
 
-=item symmetric_difference($item)
+=item symmetric_difference(...)
 
 Return a new C<Net::Silk::IPSet> containing IP addresses found in either
-this set or the union of the given I<item> list, but not both.
+this set or the union of the given I<items>, but not both.
 
-=item union($item)
+=item union(..)
 
 Return a new C<Net::Silk::IPSet> representing IP addresses present in
-either this set or the given I<item> list.
+either this set or the union of the given I<items>.
 
 =item iter()
 
-Return a sub ref iterator that returns each IP address present in
-this set. These IP addresses can also be obtained by using the
-IO operator on this object C<E<lt>$ipsetE<gt>>.
+Return a sub ref iterator that returns each IP address present in this
+set as a L<Net::Silk::IPAddr>. These IP addresses can also be obtained
+by using the IO operator on this object C<E<lt>$ipsetE<gt>>. In list
+context the iterator will produce all addresses.
 
 =item iter_cidr()
 
-Return a sub ref iterator that, upon each invocation, returns a ref to
-an array containing the IP address and prefix of each CIDR block present
-in this set.
+Return a sub ref iterator that, upon each invocation, returns
+L<Net::Silk::CIDR> object for each CIDR block present in the set. The
+iterator will procude all of them in list context.
+
+=item iter_ranges()
+
+Return a sub ref iterator which produces L<Net::Silk::Range> objects
+representing each contiguous range present in the set. The iterator will
+return all of them in list context.
 
 =back
 
 =head2 MANIPULATION METHODS
 
+A number of the methods below are for a specific type, e.g. ip address,
+wildcard, etc. If you know what you have ahead of time, these can be
+more efficient since they don't have to do class lookups to determine
+what is being added.
+
 =over
 
 =item pop()
@@ -689,7 +813,7 @@ Remove and return a random IP address from this set.
 
 =item add(...)
 
-Add all items in the provided I<item> list to this set.
+Add all items in the provided list of I<items> to this set.
 
 =item add_addr($ip)
 
@@ -716,7 +840,7 @@ Specifically add an IPwildcard (string or object) to this set.
 
 =item remove(...)
 
-Remove all items in the provided I<item> list from this set.
+Remove all items in the provided list of I<items> from this set.
 
 =item remove_addr($ip)
 
@@ -744,28 +868,33 @@ Specifically remove an IPWildcard (string or object) from this set.
 =item difference_update(...)
 
 Remove from this set all IP addresses present in the union of the
-provided I<item> list.
+provided list of I<items>.
 
-=item intersection_update($item)
+=item intersection_update(...)
 
 Remove from this set all IP addresses not present in both this set as
-well as the union of the provided I<item> list.
+well as the union of the provided list of I<items>.
 
-=item symmetric_difference_update($item)
+=item symmetric_difference_update(...)
 
 Update this set, retaining the IP addresses found in this set or
-in the union of the provided I<item> list, but not in both.
+in the union of the provided list of I<items>, but not in both.
 
-=item union_update($item)
+=item union_update(...)
 
-=item update($item)
+=item update(...)
 
-Add to this set all IP addresses present in the given I<item> list.
+Add to this set all IP addresses present in the given list of I<items>.
 
 =item clear()
 
 Remove all IP addresses from this set.
 
+=item as_bag()
+
+Return the current set as a L<Net::Silk::Bag> with counts for each ip
+address set to 1.
+
 =back
 
 =head1 OPERATORS
@@ -773,7 +902,8 @@ Remove all IP addresses from this set.
 The following operators are overloaded and work with
 C<Net::Silk::IPSet> objects:
 
-  &             ==
+  &
+  +             ==
   |             !=
   -             cmp
   ^             gt
@@ -795,11 +925,11 @@ can be tested with C<if ($ipset->{$ip}) { ... }>
 
 =head1 SEE ALSO
 
-L<Net::Silk>, L<Net::Silk::RWRec>, L<Net::Silk::IPWildcard>, L<Net::Silk::Bag>, L<Net::Silk::Pmap>, L<Net::Silk::IPAddr>, L<Net::Silk::TCPFlags>, L<Net::Silk::ProtoPort>, L<Net::Silk::File>, L<Net::Silk::Site>, L<silk(7)>
+L<Net::Silk>, L<Net::Silk::RWRec>, L<Net::Silk::Bag>, L<Net::Silk::Pmap>, L<Net::Silk::IPWildcard>, L<Net::Silk::Range>, L<Net::Silk::CIDR>, L<Net::Silk::IPAddr>, L<Net::Silk::TCPFlags>, L<Net::Silk::ProtoPort>, L<Net::Silk::File>, L<Net::Silk::Site>, L<silk(7)>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright (C) 2011-2015 by Carnegie Mellon University
+Copyright (C) 2011-2016 by Carnegie Mellon University
 
 Use of the Net-Silk library and related source code is subject to the
 terms of the following licenses:
